package com.biganto.visual.roompark.conductor.dialogs

import android.content.Context
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.ProgressBar
import androidx.annotation.LayoutRes
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.BaseRoomParkActivity
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.di.dagger.ActivityModule
import com.biganto.visual.roompark.di.dagger.AppComponent
import com.biganto.visual.roompark.di.dagger.PerScreen
import com.biganto.visual.roompark.domain.interactor.SettingsInteractor
import com.biganto.visual.roompark.domain.use_case.DownloadUseCase
import com.biganto.visual.roompark.util.view_utils.snackbar.ISnackBarProvider
import com.bluelinelabs.conductor.Controller
import com.bumptech.glide.Glide
import com.bumptech.glide.load.resource.bitmap.FitCenter
import com.google.android.material.textview.MaterialTextView
import dagger.Binds
import dagger.BindsInstance
import dagger.Component
import dagger.Module
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.schedulers.Schedulers
import jp.wasabeef.glide.transformations.BlurTransformation
import jp.wasabeef.glide.transformations.ColorFilterTransformation
import timber.log.Timber
import javax.inject.Inject


/**
 * Created by Vladislav Bogdashkin on 09.04.2019.
 */



class DownloadPlansDialogController : Controller {

    constructor():super()

    constructor(args: Bundle) : super(args)


    lateinit var progressBarDownload: ProgressBar
    lateinit var downloaderBg: ImageView
    lateinit var downloadTourTitleText: MaterialTextView
    lateinit var cancelDownloadText: MaterialTextView
    lateinit var downloadingContentText: MaterialTextView

    override fun onContextAvailable(context: Context) {
        super.onContextAvailable(context)

        DaggerPlansDownloaderScreenComponent.factory()
            .create(RoomParkApplication.component,activity as RoomParkMainActivity)
            .inject(this)
    }

    private var downloadToken = DownloadUseCase.CancellationToken(false)

    @Inject
    lateinit var downloader: SettingsInteractor

    @Inject
    lateinit var rpActivity: RoomParkMainActivity

    lateinit var snackbar: ISnackBarProvider

    private val disposables = CompositeDisposable()

    override fun onDetach(view: View) {
        disposables.clear()
        super.onDetach(view)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup): View {
        val view = inflater.inflate(getLayoutId(), container, false)

        snackbar = ActivityModule.provideSnackBar(rpActivity)

        progressBarDownload = view.findViewById(R.id.downloadProgress)
        downloaderBg = view.findViewById(R.id.backgroundDownloader)
        cancelDownloadText = view.findViewById(R.id.cancelDownloadButton)
        downloadTourTitleText = view.findViewById(R.id.tourToDownloadTitle)
        downloadingContentText = view.findViewById(R.id.downloadingTitle)
        downloadingContentText.text = resources?.getString(R.string.download_plan_types_screen_title)

//        progress.visibility = View.VISIBLE

        downloadToken = DownloadUseCase.CancellationToken(false)

        downloadTourTitleText.text = ""

        Glide.with(view)
            .load(R.drawable.flat_placeholder)
            .transform(BlurTransformation(13, 4)
                ,ColorFilterTransformation(0x99000000.toInt())
            ,FitCenter())
            .into(downloaderBg)

        disposables.add(
            downloader.downloadTourPlans(downloadToken)
//                .onErrorReturn { parseError(it);TourPreviewEntity() }
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe (
                    { model ->
                        downloadTourTitleText.text = "${model.currentProgress}/${model.totalProgress}"
                        updateProgressBar(model.currentProgress, model.totalProgress)
                        if (model.currentProgress == model.totalProgress){
                            snackbar.showSnackBar(R.string.plan_types_download_completed)
                            handleBack()
                        }
                    }
                    ,{error ->
                        Timber.e(error)
                        snackbar.showSnackBar(error.localizedMessage)
                        handleBack()
                    }
                ))
        cancelDownloadText.setOnClickListener {handleBack() }
//        downloadTour(it.tour.tour_id, downloadToken)

//        view.findViewById<View>(R.id.close_current_button).setOnClickListener { handleBack() }

        return view
    }



    private fun updateProgressBar(curr:Int, max:Int){
        activity?.runOnUiThread {
            progressBarDownload.max = max
            progressBarDownload.progress = curr
        }
    }

    @LayoutRes
    fun getLayoutId() = R.layout.download_tour_layout

    override fun handleBack(): Boolean {
        downloadToken.isCancelled = true
        return router.popController(this)
    }


}


@PerScreen
@Component(
    modules = [PlansDownloaderScreenModule::class],
    dependencies = [AppComponent::class])
interface PlansDownloaderScreenComponent {

    @Component.Factory
    interface Factory{
        fun create(
            appComponent: AppComponent
            ,@BindsInstance activity: RoomParkMainActivity
        ): PlansDownloaderScreenComponent
    }
//
//    val presenter: ArticlesScreenPresenter

    fun inject(controller: DownloadPlansDialogController)
}

@Module
abstract class PlansDownloaderScreenModule{

    @PerScreen
    @Binds
    abstract fun provideActivity(activity: RoomParkMainActivity): BaseRoomParkActivity


}
